@@ -7,7 +7,7 @@
  * 
  * Portions Copyright (C) 2000-2001 Underscore AB
  * Portions Copyright (C) 2003-2005 Quest Software, Inc.
- * Portions Copyright (C) 2004-2008 Numerous Other Contributors
+ * Portions Copyright (C) 2004-2009 Numerous Other Contributors
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -80,7 +80,10 @@
 
 
 toMarkedText::toMarkedText(QWidget *parent, const char *name)
-        : QsciScintilla(parent), toEditWidget()
+        : QsciScintilla(parent), toEditWidget(),
+        m_searchDirection(Search::SearchUndefined),
+        m_searchText(""),
+        m_searchIndicator(9) // see QsciScintilla docs
 {
     if (name)
         setObjectName(name);
@@ -90,6 +93,16 @@ toMarkedText::toMarkedText(QWidget *parent, const char *name)
     setMarginLineNumbers(0, true);
     setCallTipsStyle(CallTipsNone);
 
+    // search all occurrences
+    // allow indicator painting *under* the text (but it makes editor slower a bit...)
+    // It paints a colored box under the text for all occurrences of m_searchText.
+    SendScintilla(QsciScintilla::SCI_SETTWOPHASEDRAW, 1);
+    SendScintilla(QsciScintilla::SCI_INDICSETSTYLE, m_searchIndicator, QsciScintilla::INDIC_ROUNDBOX);
+    // TODO/FIXME: make it configurable
+    SendScintilla(QsciScintilla::SCI_INDICSETFORE, m_searchIndicator, QColor(255, 230, 90, 100));
+    SendScintilla(QsciScintilla::SCI_INDICSETUNDER, m_searchIndicator, 1);
+    // end of search all occurrences
+
     connect(this, SIGNAL(textChanged()), this, SLOT(setTextChanged()));
     connect(this, SIGNAL(copyAvailable(bool)), this, SLOT(setCopyAvailable(bool)));
     connect(this, SIGNAL(linesChanged()), this, SLOT(linesChanged()));
@@ -141,6 +154,22 @@ void toMarkedText::linesChanged()
     setMarginWidth(0, QString().fill('0', x));
 }
 
+void toMarkedText::setWordWrap(bool enable)
+{
+    if (enable)
+    {
+        setWrapMode(QsciScintilla::WrapWord);
+        setWrapVisualFlags(QsciScintilla::WrapFlagByBorder,
+                            QsciScintilla::WrapFlagByBorder);
+    }
+    else
+    {
+        setWrapMode(QsciScintilla::WrapNone);
+        setWrapVisualFlags(QsciScintilla::WrapFlagNone,
+                            QsciScintilla::WrapFlagNone);
+    }
+}
+
 void toMarkedText::setEdit(void)
 {
     if (isReadOnly())
@@ -274,6 +303,8 @@ bool toMarkedText::editOpen(QString suggestedFile)
         {
             openFilename(fname);
             emit fileOpened();
+            emit fileOpened(fname);
+            printf("emit file opened\n");
             return true;
         }
         TOCATCH
@@ -294,6 +325,7 @@ bool toMarkedText::editSave(bool askfile)
         toMainWidget()->addRecentFile(fn);
         setFilename(fn);
         setModified(false);
+        emit fileSaved(fn);
         return true;
     }
     return false;
@@ -562,7 +594,7 @@ void toMarkedText::importData(std::map<QString, QString> &data, const QString &p
     if (data[prefix + ":Edited"].isEmpty())
         setModified(false);
 }
-
+/*
 static int FindIndex(const QString &str, int line, int col)
 {
     int pos = 0;
@@ -574,7 +606,7 @@ static int FindIndex(const QString &str, int line, int col)
         pos++;
     }
     return pos + col;
-}
+}*/
 
 void toMarkedText::findPosition(int index, int &line, int &col)
 {
@@ -595,34 +627,105 @@ void toMarkedText::findPosition(int index, int &line, int &col)
     return ;
 }
 
-bool toMarkedText::searchNext(toSearchReplace *search)
+bool toMarkedText::searchNext(const QString & text)
 {
-    QString text = toMarkedText::text();
+    return findText(Search::SearchForward);
+}
 
-    int col;
-    int line;
-    getCursorPosition(&line, &col);
-    int pos = FindIndex(text, line, col) + 1;
+bool toMarkedText::searchPrevious(const QString & text)
+{
+    return findText(Search::SearchBackward);
+}
 
-    int endPos;
-    if (search->findString(text, pos, endPos))
+bool toMarkedText::findText(Search::SearchDirection direction)
+{
+    bool r = false;
+    int line, index;
+    Search::SearchDirection state = m_searchDirection;
+
+    getCursorPosition(&line, &index);
+
+    if (m_searchText != toMainWidget()->searchDialog()->currentSearchText())
     {
-        int endCol;
-        int endLine;
-        findPosition(pos, line, col);
-        findPosition(endPos, endLine, endCol);
-        setSelection(line, col, endLine, endCol);
-        ensureCursorVisible();
-        return true;
+        m_searchText = toMainWidget()->searchDialog()->currentSearchText();
+        state = Search::SearchUndefined; // to reset the findFirst() condition
     }
 
-    return false;
+    if (m_searchDirection != direction)
+        m_searchDirection = direction;
+
+    if (m_searchDirection != state)
+    {
+        r = findFirst(m_searchText,
+                      toMainWidget()->searchDialog()->searchMode() == Search::SearchRegexp,
+                      toMainWidget()->searchDialog()->caseSensitive(),
+                      toMainWidget()->searchDialog()->wholeWords(),
+                      true, //bool   wrap,
+                      (m_searchDirection == Search::SearchForward),
+                      line,
+                      index,
+                      true //bool   show = true
+                     );
+        // find and highlight all occurrences of m_searchText
+        // from the beginning to the end
+        int from = 0;
+        int to = text().length();
+
+        SendScintilla(QsciScintilla::SCI_SETINDICATORCURRENT, m_searchIndicator);
+        // clear previously used marked text
+        SendScintilla(QsciScintilla::SCI_INDICATORCLEARRANGE, 0, to);
+
+        // set searching flags
+        int searchFlags = 0;
+        //QsciScintilla::SCFIND_WORDSTART;
+        if (toMainWidget()->searchDialog()->searchMode() == Search::SearchRegexp)
+            searchFlags |= QsciScintilla::SCFIND_REGEXP;
+        if (toMainWidget()->searchDialog()->caseSensitive())
+            searchFlags |= QsciScintilla::SCFIND_MATCHCASE;
+        if (toMainWidget()->searchDialog()->wholeWords())
+            searchFlags |= QsciScintilla::SCFIND_WHOLEWORD;
+
+        while (from < to)
+        {
+            // set searching range
+            SendScintilla(QsciScintilla::SCI_SETTARGETSTART, from);
+            SendScintilla(QsciScintilla::SCI_SETTARGETEND, to);
+            SendScintilla(QsciScintilla::SCI_SETSEARCHFLAGS, searchFlags);
+            from = SendScintilla(QsciScintilla::SCI_SEARCHINTARGET,
+                                m_searchText.length(), m_searchText.toUtf8().data());
+
+            // SCI_SEARCHINTARGET returns -1 when it doesn't find anything
+            if (from == -1)
+                break;
+
+            int end = SendScintilla(QsciScintilla::SCI_GETTARGETEND);
+            // mark current occurrence of searchText
+            SendScintilla(QsciScintilla::SCI_INDICATORFILLRANGE, from, end - from);
+            from = end;
+        }
+    }
+    else
+        findNext();
+
+    return r;
 }
 
 void toMarkedText::searchReplace(const QString &newData)
 {
-    if (!isReadOnly())
-        insert(newData);
+    if (searchCanReplace(false))
+//         insert(newData);
+        replace(newData);
+}
+
+void toMarkedText::searchReplaceAll(const QString & newData)
+{
+    if (!searchCanReplace(true))
+        return;
+
+    QsciScintilla::beginUndoAction();
+    while (findNext())
+        replace(newData);
+    QsciScintilla::endUndoAction();
 }
 
 bool toMarkedText::searchCanReplace(bool all)
@@ -663,6 +766,16 @@ void toMarkedText::insert(const QString &str, bool select)
     QsciScintilla::endUndoAction();
 }
 
+#if 0
+// TODO: this part is waiting for QScintilla backend feature (yet unimplemented).
+void toMarkedText::setSelectionType(int aType)
+{
+    qDebug() << "setSelectionType" << SendScintilla(SCI_GETSELECTIONMODE) << aType;
+    qDebug() << SendScintilla(SCI_SETSELECTIONMODE, aType);
+    qDebug() << "setSelectionType" << SendScintilla(SCI_GETSELECTIONMODE) << aType;
+}
+#endif
+
 void toMarkedText::setTextChanged()
 {
     redoEnabled(isRedoAvailable());
